class CoordinatorNotRegisteredInMetaPersister(Exception): ...


class AssetNotRegisteredInMetaPersister(Exception): ...


class AssetNotRegisteredInDataPersister(Exception): ...


class AssetNotFoundException(Exception): ...


class MetaDataNotStoredException(Exception): ...
